@@ -27,7 +27,6 @@ python_token_re = re.compile(PseudoToken)
 
 SET_LOCAL = 0
 SET_GLOBAL = 1
-SET_MODULE = 2
 
 identchars = string.ascii_letters + '_'
 namechars = identchars + string.digits
@@ -64,7 +63,7 @@ closurePairsRev = {'(': ')', '[': ']', '{': '}'}
 
 
 # Used for #set global
-Components = collections.namedtuple('Components', ['LVALUE', 'OP', 'RVALUE'])
+Components = collections.namedtuple('Components', ['lvalue', 'op', 'rvalue'])
 
 
 tripleQuotedStringREs = {}
@@ -281,12 +280,12 @@ class _LowLevelParser(SourceReader):
             escCharLookBehind +
             re.escape(self.setting('cheetahVarStartToken'))
             + validCharsLookAhead
-            )
+        )
 
         self.cheetahVarInExpressionStartTokenRE = re.compile(
             re.escape(self.setting('cheetahVarStartToken'))
             + r'(?=[A-Za-z_])'
-            )
+        )
 
         self.expressionPlaceholderStartRE = re.compile(
             escCharLookBehind +
@@ -665,10 +664,6 @@ class _LowLevelParser(SourceReader):
         argList = ArgList()
         onDefVal = False
 
-        # @@TR: this settings mangling should be removed
-        useNameMapper_orig = self.setting('useNameMapper')
-        self.setSetting('useNameMapper', False)
-
         while True:
             if self.atEnd():
                 raise ParseError(
@@ -689,21 +684,15 @@ class _LowLevelParser(SourceReader):
                 argList.next()
                 onDefVal = False
                 self.advance()
-            elif self.startswith(self.cheetahVarStartToken) and not onDefVal:
-                self.advance(len(self.cheetahVarStartToken))
+            elif self.startswith(self.cheetahVarStartToken):
+                raise ParseError(self, '$ is not allowed here.')
             elif self.matchIdentifier() and not onDefVal:
                 argList.add_argument(self.getIdentifier())
             elif onDefVal:
-                if self.matchCheetahVarInExpressionStartToken():
-                    token = self.getCheetahVar()
-                elif self.matchCheetahVarStart():
-                    # it has syntax that is only valid at the top level
-                    self._raiseErrorAboutInvalidCheetahVarSyntaxInExpr()
-                else:
-                    token = self.getPyToken()
-                    if token in ('{', '(', '['):
-                        self.rev()
-                        token = self.getExpression(enclosed=True)
+                token = self.getPyToken()
+                if token in ('{', '(', '['):
+                    self.rev()
+                    token = self.getExpression(enclosed=True)
                 argList.add_default(token)
             elif c == '*' and not onDefVal:
                 varName = self.getc()
@@ -716,7 +705,6 @@ class _LowLevelParser(SourceReader):
             else:
                 raise ParseError(self, 'Unexpected character.')
 
-        self.setSetting('useNameMapper', useNameMapper_orig)  # @@TR: see comment above
         return argList.merge()
 
     def getExpressionParts(
@@ -838,6 +826,17 @@ class _LowLevelParser(SourceReader):
             pyTokensToBreakAt=pyTokensToBreakAt,
             useNameMapper=useNameMapper))
 
+    def get_python_expression(self, failure_msg, **kwargs):
+        """Get an expression that should not contain cheetah variables.
+        Raises a ParseError with `failure_msg` on failure.
+        """
+        expr_pos = self.pos()
+        expr = self.getExpression(**kwargs)
+        if 'VFN(' in expr or 'VFFSL(' in expr:
+            self.setPos(expr_pos)
+            raise ParseError(self, failure_msg)
+        return expr
+
     def _raiseErrorAboutInvalidCheetahVarSyntaxInExpr(self):
         match = self.matchCheetahVarStart()
         groupdict = match.groupdict()
@@ -890,6 +889,7 @@ class LegacyParser(_LowLevelParser):
     sending state dependent code generation commands to
     Cheetah.legacy_compiler.LegacyCompiler
     """
+
     def __init__(self, src, compiler=None):
         super(LegacyParser, self).__init__(src)
         self.setSettingsManager(compiler)
@@ -1241,17 +1241,15 @@ class LegacyParser(_LowLevelParser):
         self.advance(len('attr'))
         self.getWhiteSpace()
         if self.matchCheetahVarStart():
-            self.getCheetahVarStartToken()
+            raise ParseError(self, '#attr directive must not contain `$`')
         attribName = self.getIdentifier()
         self.getWhiteSpace()
         self.getAssignmentOperator()
-        expr = self.getExpression()
-        if 'VFN(' in expr or 'VFFSL(' in expr:
-            raise ParseError(
-                self,
-                'Invalid #attr directive. '
-                'It should contain simple Python literals.'
-            )
+        self.getWhiteSpace()
+        expr = self.get_python_expression(
+            'Invalid #attr directive. '
+            'It should contain simple Python literals.'
+        )
         self._compiler.addAttribute(attribName, expr)
         self._eatRestOfDirectiveTag(isLineClearToStartToken, endOfFirstLinePos)
 
@@ -1259,27 +1257,28 @@ class LegacyParser(_LowLevelParser):
         isLineClearToStartToken = self.isLineClearToStartToken()
         endOfFirstLinePos = self.findEOL()
         self.getDirectiveStartToken()
-        decoratorExpr = self.getExpression()
-        self._compiler.addDecorator(decoratorExpr)
+        decorator_expr = self.getExpression()
+        if decorator_expr in ('@classmethod', '@staticmethod'):
+            self.setPos(self.pos() - len(decorator_expr))
+            raise ParseError(
+                self, '@classmethod / @staticmethod are not supported',
+            )
+        self._compiler.addDecorator(decorator_expr)
         self._eatRestOfDirectiveTag(isLineClearToStartToken, endOfFirstLinePos)
         self.getWhiteSpace()
 
         directiveName = self.matchDirective()
         if not directiveName or directiveName not in ('def', 'block', '@'):
             raise ParseError(
-                self, msg='Expected #def, #block or another @decorator')
+                self, 'Expected #def, #block or another @decorator',
+            )
         self.eatDirective()
 
     def eatDef(self):
         self._eatDefOrBlock('def')
 
     def eatBlock(self):
-        startPos = self.pos()
-        methodName, rawSignature = self._eatDefOrBlock('block')
-        self._compiler._blockMetaData[methodName] = {
-            'raw': rawSignature,
-            'lineCol': self.getRowCol(startPos),
-        }
+        self._eatDefOrBlock('block')
 
     def _eatDefOrBlock(self, directiveName):
         assert directiveName in ('def', 'block')
@@ -1290,20 +1289,34 @@ class LegacyParser(_LowLevelParser):
         self.advance(len(directiveName))
         self.getWhiteSpace()
         if self.matchCheetahVarStart():
-            self.getCheetahVarStartToken()
+            raise ParseError(self, 'use #def func() instead of #def $func()')
         methodName = self.getIdentifier()
         self.getWhiteSpace()
-        if self.peek() == '(':
+
+        if directiveName == 'block' and self.peek() == '(':
+            raise ParseError(
+                self, '#block must not have an argspec, did you mean #def?',
+            )
+
+        if directiveName == 'def' and self.peek() != '(':
+            raise ParseError(self, '#def must contain an argspec (at least ())')
+
+        if directiveName == 'def':
+            arglist_position = self.pos()
             argsList = self.getDefArgList()
-            self.advance()              # past the closing ')'
+            self.advance()  # Past closing ')'
             if argsList and argsList[0][0] == 'self':
-                del argsList[0]
+                # So the exception points at the right place
+                self.setPos(arglist_position + 1)
+                raise ParseError(
+                    self,
+                    'Do not specify `self` in an arglist, it is assumed',
+                )
         else:
             argsList = []
 
         if self.matchColonForSingleLineShortFormDirective():
             self.getc()
-            rawSignature = self[startPos:endOfFirstLinePos]
             self._eatSingleLineDef(
                 methodName=methodName,
                 argsList=argsList,
@@ -1321,7 +1334,6 @@ class LegacyParser(_LowLevelParser):
             if self.peek() == ':':
                 self.getc()
             self.pushToOpenDirectivesStack(directiveName)
-            rawSignature = self[startPos:self.pos()]
             self._eatMultiLineDef(
                 methodName=methodName,
                 argsList=argsList,
@@ -1329,8 +1341,6 @@ class LegacyParser(_LowLevelParser):
                 isLineClearToStartToken=isLineClearToStartToken,
             )
 
-        return methodName, rawSignature
-
     def _eatMultiLineDef(self, methodName, argsList, startPos, isLineClearToStartToken=False):
         self.getExpression()  # slurp up any garbage left at the end
         signature = self[startPos:self.pos()]
@@ -1416,20 +1426,18 @@ class LegacyParser(_LowLevelParser):
             self.getIdentifier()
             self.getWhiteSpace()
             style = SET_GLOBAL
-        elif self.startswith('module'):
-            self.getIdentifier()
-            self.getWhiteSpace()
-            style = SET_MODULE
 
-        LVALUE = self.getExpression(pyTokensToBreakAt=assignmentOps, useNameMapper=False).strip()
-        OP = self.getAssignmentOperator()
-        RVALUE = self.getExpression()
-        expr = LVALUE + ' ' + OP + ' ' + RVALUE.strip()
+        lvalue = self.get_python_expression(
+            'lvalue of #set cannot contain `$`',
+            pyTokensToBreakAt=assignmentOps,
+        ).strip()
+        op = self.getAssignmentOperator()
+        rvalue = self.getExpression()
 
         self._eatRestOfDirectiveTag(isLineClearToStartToken, endOfFirstLine)
 
-        expr_components = Components(LVALUE, OP, RVALUE)
-        self._compiler.addSet(expr, expr_components, style)
+        expr_components = Components(lvalue, op, rvalue)
+        self._compiler.addSet(expr_components, style)
 
     def eatSlurp(self):
         if self.isLineClearToStartToken():
@@ -1445,8 +1453,9 @@ class LegacyParser(_LowLevelParser):
         macro = self._macros[macroName]
 
         self.getWhiteSpace()
-        args = self.getExpression(
-            useNameMapper=False, pyTokensToBreakAt=[':']
+        args = self.get_python_expression(
+            'Macro arguments must not contain a `$`',
+            pyTokensToBreakAt=[':'],
         ).strip()
 
         if self.matchColonForSingleLineShortFormDirective():
@@ -1485,7 +1494,6 @@ class LegacyParser(_LowLevelParser):
         self.setPos(origPos)
 
     def eatCall(self):
-        # @@TR: need to enable single line version of this
         isLineClearToStartToken = self.isLineClearToStartToken()
         endOfFirstLinePos = self.findEOL()
         lineCol = self.getRowCol()
@@ -1576,18 +1584,15 @@ class LegacyParser(_LowLevelParser):
         assert directiveName in self._closeableDirectives
         self._openDirectivesStack.append(directiveName)
 
-    def popFromOpenDirectivesStack(self, directiveName):
+    def popFromOpenDirectivesStack(self, directive_name):
         if not self._openDirectivesStack:
             raise ParseError(self, msg="#end found, but nothing to end")
 
-        if self._openDirectivesStack[-1] == directiveName:
-            del self._openDirectivesStack[-1]
-        else:
+        last = self._openDirectivesStack.pop()
+        if last != directive_name:
             raise ParseError(
                 self,
-                msg="#end %s found, expected #end %s" % (
-                    directiveName, self._openDirectivesStack[-1]
-                )
+                '#end {0} found, expected #end {1}'.format(directive_name, last)
             )
 
     def assertEmptyOpenDirectivesStack(self):
